<?php

/**
 * Plugin Name: Sirv
 * Plugin URI: http://sirv.com
 * Description: Instantly resize or crop images to any size. Add watermarks, titles, text and image effects. Embed them as images, galleries, zooms or 360 spins. Serve them from the fast CDN. Responsive, to perfectly fit the screen. Use the "Add Sirv Media" button on posts and pages. <a href="admin.php?page=sirv/sirv/options.php">Settings</a>
 * Version: 3.0
 * Author: sirv.com
 * Author URI: sirv.com
 * License: GPLv2
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

ini_set('max_execution_time', 600);

define('SIRV_PLUGIN_PATH', str_replace('/sirv.php', '', plugin_basename(__FILE__)));
define('SIRV_PLUGIN_PATH_WITH_SLASH', '/' . SIRV_PLUGIN_PATH);

global $sirv_plugin_version;
global $sirv_make_responsive;

$sirv_plugin_version = '3.0';


function sirv_debug_msg($msg){
    $path = realpath(dirname(__FILE__));
    $fn = fopen($path . '/debug.txt', 'a+');
    if(is_array($msg)){
      fwrite($fn, print_r($msg, true) . PHP_EOL);
    }else{
      fwrite($fn, $msg . PHP_EOL);
    }

    fclose($fn);
}


// load shortcodes
require_once (dirname (__FILE__) . '/sirv/shortcodes.php');

//create shortcode's table on plugin activate
register_activation_hook( __FILE__, 'sirv_activation_callback' );

function sirv_activation_callback(){

    set_transient( 'isSirvActivated', true, 30 );
    sirv_create_plugin_tables();
    sirv_register_settings();

    sirv_upgrade_plugin();

    $notices= get_option('sirv_admin_notices', array());
    $notices[]= 'Congratulations, you\'ve just installed Sirv for WordPress! Now <a href="admin.php?page='. SIRV_PLUGIN_PATH .'/sirv/options.php">configure the Sirv plugin</a> to start using it.';
    update_option('sirv_admin_notices', $notices);
}


add_action( 'plugins_loaded', 'sirv_upgrade_plugin' );
function sirv_upgrade_plugin(){
  global $sirv_plugin_version;
  $sirv_plugin_version_installed = get_option('WP_SIRV_VERSION_PLUGIN_INSTALLED');


  if(empty($sirv_plugin_version_installed) || $sirv_plugin_version_installed != $sirv_plugin_version){

    global $wpdb;

    $shortcodes_t = $wpdb->prefix . 'sirv_shortcodes';

    $t_structure = $wpdb->get_results( "DESCRIBE $shortcodes_t", ARRAY_A);
    $t_fields = sirv_get_field_names($t_structure);

    if(!in_array('shortcode_options', $t_fields)){
        $wpdb->query("ALTER TABLE $shortcodes_t ADD COLUMN shortcode_options TEXT NOT NULL after images");
    }

    if(!in_array('timestamp', $t_fields)){
        //$wpdb->query("ALTER TABLE $shortcodes_t ADD COLUMN shortcode_options TEXT NOT NULL after images");
        $wpdb->query("ALTER TABLE $shortcodes_t ADD COLUMN timestamp DATETIME NULL DEFAULT NULL AFTER shortcode_options");
    }

    if(!sirv_is_unique_field('attachment_id')){
      sirv_set_unique_field('attachment_id');
    }

    update_option('WP_SIRV_VERSION_PLUGIN_INSTALLED', $sirv_plugin_version);
  }
}


function sirv_get_field_names($data){
  $tmp_arr = Array();

  foreach ($data as $key => $field_data) {
    $tmp_arr[] = $field_data['Field'];
  }

  return $tmp_arr;
}


add_action( 'wp_head', 'sirv_meta_head', 10);

function sirv_meta_head(){

  $sirv_url = sirv_build_url();

  echo '<link rel="preconnect" href="'. $sirv_url .'" crossorigin/>'. PHP_EOL;
  echo '<link rel="dns-prefetch" href="'. $sirv_url .'"/>'. PHP_EOL;

  echo '<link rel="preconnect" href="https://scripts.sirv.com" crossorigin/>' . PHP_EOL;
  echo '<link rel="dns-prefetch" href="https://scripts.sirv.com"/>' . PHP_EOL;
}


function sirv_build_url(){
  $bucket = get_option('AWS_BUCKET');
  $network_type = get_option('WP_SIRV_NETWORK');
  $protocol = 'https://';
  $sirv_url = '';

  switch ($network_type) {
    case 1:
      $sirv_url = $protocol . $bucket . '-cdn.sirv.com';
      break;
    case 2:
      $sirv_url = $protocol . $bucket . '.sirv.com';
      break;
    case 3:
      $sirv_url = get_option('WP_SIRV_CUSTOM_DOMAIN') !== '' ? get_option('WP_SIRV_CUSTOM_DOMAIN') : $protocol . $bucket . '.sirv.com';
      break;
  }

  return $sirv_url;
}


if( function_exists('register_block_type' ) ){
  if( !function_exists('sirv_addmedia_block')){
    function sirv_addmedia_block(){

      wp_register_script(
        'sirv-addmedia-block-editor-js',
        plugins_url('/sirv/gutenberg/addmedia-block/editor-script.js', __FILE__),
        array( 'wp-blocks', 'wp-element', 'wp-editor', 'sirv_modal', 'sirv_logic', 'sirv_modal-logic', 'sirv_logic-md5', 'jquery')
      );

      wp_register_style(
        'sirv-addmedia-block-css',
        plugins_url( '/sirv/gutenberg/addmedia-block/style.css', __FILE__ ),
        array( 'wp-edit-blocks' ),
        filemtime( plugin_dir_path( __FILE__ ) . 'sirv/gutenberg/addmedia-block/style.css' )
      );

      wp_register_style(
        'sirv-addmedia-block-editor-css',
        plugins_url( '/sirv/gutenberg/addmedia-block/editor-style.css', __FILE__ ),
        array( 'wp-edit-blocks' ),
        filemtime( plugin_dir_path( __FILE__ ) . 'sirv/gutenberg/addmedia-block/editor-style.css' )
      );

      register_block_type( 'sirv/addmedia-block', array(
          'editor_script' => 'sirv-addmedia-block-editor-js',
          'editor_style'  => 'sirv-addmedia-block-editor-css',
          'style'         => 'sirv-addmedia-block-css',
          //'render_callback' => 'sirv_frontend_render_add_media_block',
      ) );
    }

    add_action( 'init', 'sirv_addmedia_block' );
  }

}


function sirv_frontend_render_add_media_block($attributes, $content){
  print_r($attributes);
  return sprintf(
          '[sirv-gallery id=%1$s]',
           $attributes['shId']
         );
}


//show message on plugin activation
add_action('admin_notices', 'sirv_admin_notices');

function sirv_admin_notices() {
  if ($notices= get_option('sirv_admin_notices')) {
    foreach ($notices as $notice) {
      echo "<div class='updated'><p>$notice</p></div>";
    }
    delete_option('sirv_admin_notices');
  }

  //show review notice
  $sirv_review_notice = get_option('sirv_review_notice');
  if(empty($sirv_review_notice)){
    update_option('sirv_review_notice', time());
    $sirv_review_notice = NULL;
  }
  if(is_numeric($sirv_review_notice)){
    $noticed_time = (int) $sirv_review_notice;
    $fire_time = $noticed_time + (14 * 24 * 60 * 60);
    if(time() >= $fire_time){
      wp_enqueue_script( 'sirv_review', plugins_url('/sirv/js/wp-sirv-review.js', __FILE__), array(), '1.0.0');

      echo '<div class="notice notice-info is-dismissible"><p>We noticed you\'ve been using Sirv for some time now - we hope you love it! We\'d be thrilled if you could <a target="_blank" href="https://wordpress.org/support/plugin/sirv/reviews/">give us a 5-star rating on WordPress.org!</a></p>
        <p>As a thank you, we\'ll give you 1GB extra free storage (regardless of the rating you choose).</p>
        <p>If you need help with the Sirv plugin, please <a href="admin.php?page='. SIRV_PLUGIN_PATH .'/sirv/options.php#help-feedback">contact our team</a> and we\'ll reply ASAP.</p></div>';
      //update_option('sirv_review_notice', 'noticed');
    }
  }
}


function sirv_create_plugin_tables(){
    global $wpdb;

    $table_shortcodes_name = $wpdb->prefix . 'sirv_shortcodes';
    $sql_shortcodes = "CREATE TABLE $table_shortcodes_name (
      id int unsigned NOT NULL auto_increment,
      width varchar(20) DEFAULT 'auto',
      thumbs_height varchar(20) DEFAULT NULL,
      gallery_styles varchar(255) DEFAULT NULL,
      align varchar(30) DEFAULT '',
      profile varchar(100) DEFAULT 'false',
      link_image varchar(10) DEFAULT 'false',
      show_caption varchar(10) DEFAULT 'false',
      use_as_gallery varchar(10) DEFAULT 'false',
      use_sirv_zoom varchar(10) DEFAULT 'false',
      images text DEFAULT NULL,
      shortcode_options text NOT NULL,
      timestamp datetime DEFAULT NULL,
      PRIMARY KEY  (id))
      ENGINE=MyISAM DEFAULT CHARSET=utf8;";

    $table_images_name = $wpdb->prefix . 'sirv_images';
    $sql_sirv_images = "CREATE TABLE $table_images_name (
      id int unsigned NOT NULL auto_increment,
      attachment_id int(11) NOT NULL,
      wp_path varchar(255) DEFAULT NULL,
      size int(10) DEFAULT NULL,
      sirvpath varchar(255) DEFAULT NULL,
      sirv_image_url varchar(255) DEFAULT NULL,
      sirv_folder varchar(255) DEFAULT NULL,
      timestamp datetime DEFAULT NULL,
      timestamp_synced datetime DEFAULT NULL,
      PRIMARY KEY (id),
      UNIQUE KEY `unique_key` (attachment_id)
      )ENGINE=InnoDB DEFAULT CHARSET=utf8;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $sql_shortcodes );
    dbDelta( $sql_sirv_images );
}

register_deactivation_hook(__FILE__, 'sirv_deactivation_callback');

function sirv_deactivation_callback(){
  //some code here
}


$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 'sirv_plugin_settings_link' );

function sirv_plugin_settings_link($links) {
  $settings_link = '<a href="admin.php?page='. SIRV_PLUGIN_PATH .'/sirv/options.php">Settings</a>';
  array_unshift($links, $settings_link);
  return $links;
}


//add button Sirv Media near Add Media
add_action('media_buttons','sirv_button', 11);

function sirv_button($editor_id = 'content') {
    /*wp_register_style('sirv_style', plugins_url('/sirv/css/wp-sirv.css', __FILE__));
    wp_enqueue_style('sirv_style');
    wp_enqueue_script( 'sirv_logic', plugins_url('/sirv/js/wp-sirv.js', __FILE__), array( 'jquery', 'jquery-ui-sortable' ), '1.0.0');
    wp_localize_script( 'sirv_logic', 'ajax_object', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'assets_path' => plugins_url('/sirv/assets', __FILE__)) );
    wp_enqueue_script( 'sirv_logic-md5', plugins_url('/sirv/js/wp-sirv-md5.min.js', __FILE__), array(), '1.0.0');
    wp_enqueue_script( 'sirv_modal', plugins_url('/sirv/js/wp-sirv-bpopup.min.js', __FILE__), array('jquery'), '1.0.0');
    wp_enqueue_script( 'sirv_modal-logic', plugins_url('/sirv/js/wp-sirv-modal.js', __FILE__), array('jquery'), '1.0.0');

    $isNotEmptySirvOptions = sirv_check_empty_options_on_backend();
    wp_localize_script( 'sirv_modal-logic', 'modal_object', array('media_add_url' =>  plugins_url('/sirv/templates/media_add.html', __FILE__), 'login_error_url' => plugins_url('/sirv/templates/login_error.html', __FILE__), 'featured_image_url' => plugins_url('/sirv/templates/featured_image.html', __FILE__), 'isNotEmptySirvOptions' => $isNotEmptySirvOptions));
    wp_enqueue_script('sirv-shortcodes-page', plugins_url('/sirv/js/wp-sirv-shortcodes-page.js', __FILE__), array( 'jquery'), '1.0.0');*/


    echo '<a href="#" class="button sirv-modal-click" title="Sirv add/insert images"><span class="dashicons dashicons-format-gallery" style="padding-top: 2px;"></span> Add Sirv Media</a><div class="sirv-modal"><div class="modal-content"></div></div>';
}


function sirv_check_empty_options_on_backend(){

    require_once 'sirv/options-service.php';

    $host = getValue::getOption('AWS_HOST');
    $bucket = getValue::getOption('AWS_BUCKET');
    $key = getValue::getOption('AWS_KEY');
    $secret_key = getValue::getOption('AWS_SECRET_KEY');

    if(empty($host) || empty($bucket) || empty($key) || empty($secret_key)){
        return false;
    }else{
        return true;
    }
}


//create menu for wp plugin and register settings
add_action("admin_menu", "sirv_create_menu", 0);

function sirv_create_menu(){
    $settings_item = SIRV_PLUGIN_PATH_WITH_SLASH .'/sirv/options.php';
    $library_item = SIRV_PLUGIN_PATH_WITH_SLASH .'/sirv/media_library.php';
    $shortcodes_view_item = SIRV_PLUGIN_PATH_WITH_SLASH .'/sirv/shortcodes-view.php';
    $stats = 'admin.php?page='. SIRV_PLUGIN_PATH .'/sirv/options.php#sirv-stats';

    add_menu_page('Sirv Menu', 'Sirv', 'manage_options', $settings_item, NULL, plugins_url(SIRV_PLUGIN_PATH_WITH_SLASH .'/sirv/assets/icon.png'));
    add_submenu_page( $settings_item, 'Sirv Settings', 'Settings', 'manage_options', $settings_item);
    add_submenu_page( $settings_item, 'Sirv Shortcodes', 'Shortcodes', 'manage_options', $shortcodes_view_item );
    add_submenu_page( $settings_item, 'Sirv Media Library', 'Media Library', 'manage_options', $library_item);
    //add_submenu_page( $settings_item, 'Sirv Stats', 'Stats', 'manage_options', $stats);
}


add_action('admin_enqueue_scripts', 'sirv_admin_scripts', 20);
function sirv_admin_scripts(){
    //if(!is_admin() || empty($_GET['page'])) return;
    if(!is_admin()) return;

    global $pagenow;

    if( in_array($pagenow, array('post-new.php', 'post.php')) /*&& $_GET['action'] === 'edit'*//* && function_exists('register_block_type')*/){
      wp_register_style('sirv_style', plugins_url('/sirv/css/wp-sirv.css', __FILE__));
      wp_enqueue_style('sirv_style');
      wp_register_style('sirv_mce_style', plugins_url('/sirv/css/wp-sirv-shortcode-view.css', __FILE__));
      wp_enqueue_style('sirv_mce_style');
      wp_enqueue_script( 'sirv_logic', plugins_url('/sirv/js/wp-sirv.js', __FILE__), array( 'jquery', 'jquery-ui-sortable' ), '1.0.0');
      wp_localize_script( 'sirv_logic', 'ajax_object', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'assets_path' => plugins_url('/sirv/assets', __FILE__), 'plugin_path' => SIRV_PLUGIN_PATH ) );
      wp_enqueue_script( 'sirv_logic-md5', plugins_url('/sirv/js/wp-sirv-md5.min.js', __FILE__), array(), '1.0.0');
      wp_enqueue_script( 'sirv_modal', plugins_url('/sirv/js/wp-sirv-bpopup.min.js', __FILE__), array('jquery'), '1.0.0');
      //wp_enqueue_script( 'sirv_modal-logic', plugins_url('/sirv/js/wp-sirv-modal.js', __FILE__), array('jquery', 'sirv-addmedia-block-editor-js'), '1.0.0');
      wp_enqueue_script( 'sirv_modal-logic', plugins_url('/sirv/js/wp-sirv-modal.js', __FILE__), array('jquery'), '1.0.0');

      $isNotEmptySirvOptions = sirv_check_empty_options_on_backend();
      wp_localize_script( 'sirv_modal-logic', 'modal_object', array('media_add_url' =>  plugins_url('/sirv/templates/media_add.html', __FILE__), 'login_error_url' => plugins_url('/sirv/templates/login_error.html', __FILE__), 'featured_image_url' => plugins_url('/sirv/templates/featured_image.html', __FILE__), 'isNotEmptySirvOptions' => $isNotEmptySirvOptions));
      wp_enqueue_script('sirv-shortcodes-page', plugins_url('/sirv/js/wp-sirv-shortcodes-page.js', __FILE__), array( 'jquery'), '1.0.0');
    }

    if(isset($_GET['page']) && $_GET['page'] == SIRV_PLUGIN_PATH .'/sirv/options.php'){
      wp_enqueue_script( 'sirv_scrollspy', plugins_url('/sirv/js/scrollspy.js', __FILE__), array('jquery'), '1.0.0');
    }

    if(isset($_GET['page']) && $_GET['page'] == SIRV_PLUGIN_PATH .'/sirv/shortcodes-view.php'){
      wp_register_style('sirv_style', plugins_url('/sirv/css/wp-sirv.css', __FILE__));
      wp_enqueue_style('sirv_style');
      wp_enqueue_script( 'sirv_logic', plugins_url('/sirv/js/wp-sirv.js', __FILE__), array( 'jquery', 'jquery-ui-sortable' ), '1.0.0');
      wp_localize_script( 'sirv_logic', 'ajax_object', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'assets_path' => plugins_url('/sirv/assets', __FILE__)) );
      wp_enqueue_script( 'sirv_logic-md5', plugins_url('/sirv/js/wp-sirv-md5.min.js', __FILE__), array(), '1.0.0');
      wp_enqueue_script( 'sirv_modal', plugins_url('/sirv/js/wp-sirv-bpopup.min.js', __FILE__), array('jquery'), '1.0.0');

      wp_localize_script( 'sirv_modal', 'modal_object', array('media_add_url' =>  plugins_url('/sirv/templates/media_add.html', __FILE__), 'login_error_url' => plugins_url('/sirv/templates/login_error.html', __FILE__), 'featured_image_url' => plugins_url('/sirv/templates/featured_image.html', __FILE__)));

      wp_register_script('sirv-shortcodes-page',plugins_url('/sirv/js/wp-sirv-shortcodes-page.js', __FILE__), array( 'jquery'), '1.0.0');
      wp_enqueue_script('sirv-shortcodes-page');
      wp_localize_script( 'sirv-shortcodes-page', 'sirvShortcodeObject', array('isShortcodesPage' => true));
    }
}


//include plugin for tinyMCE to show sirv gallery shortcode in visual mode
add_filter('mce_external_plugins', 'sirv_tinyMCE_plugin_shortcode_view');

function sirv_tinyMCE_plugin_shortcode_view () {
     return array('sirvgallery' => plugins_url('sirv/js/wp-sirv-shortcode-view.js', __FILE__));
}


add_action('admin_init', 'sirv_admin_init');
function sirv_admin_init(){
    sirv_tinyMCE_plugin_shortcode_view_styles();
    sirv_redirect_to_options();
}


//add styles for tinyMCE plugin
function sirv_tinyMCE_plugin_shortcode_view_styles(){
    add_editor_style( plugins_url('/sirv/css/wp-sirv-shortcode-view.css', __FILE__) );
}

//redirect to options after activate plugin
function sirv_redirect_to_options() {
  // Bail if no activation redirect
    if ( ! get_transient( 'isSirvActivated' ) ) {
    return;
  }

  // Delete the redirect transient
  delete_transient( 'isSirvActivated' );

  // Bail if activating from network, or bulk
  if ( is_network_admin() || isset( $_GET['activate-multi'] ) ) {
    return;
  }

  if(get_option('AWS_BUCKET') == '' || get_option('AWS_KEY') == '' || get_option('AWS_SECRET_KEY') == ''){
    // Redirect to bbPress about page
    wp_safe_redirect( add_query_arg( array( 'page' => SIRV_PLUGIN_PATH .'/sirv/options.php' ), admin_url( 'admin.php' ) ) );
  }

}


function sirv_register_settings(){
    register_setting( 'sirv-settings-group', 'AWS_KEY' );
    register_setting( 'sirv-settings-group', 'AWS_SECRET_KEY' );
    register_setting( 'sirv-settings-group', 'AWS_HOST' );
    register_setting( 'sirv-settings-group', 'AWS_BUCKET' );
    register_setting( 'sirv-settings-group', 'WP_FOLDER_ON_SIRV');
    register_setting( 'sirv-settings-group', 'WP_USE_SIRV_CDN');
    register_setting( 'sirv-settings-group', 'WP_SIRV_NETWORK');
    register_setting( 'sirv-settings-group', 'WP_SIRV_CUSTOM_DOMAIN');

    register_setting( 'sirv-settings-group', 'WP_SIRV_SHORTCODES_PROFILES');
    register_setting( 'sirv-settings-group', 'WP_SIRV_CDN_PROFILES');
    register_setting( 'sirv-settings-group', 'WP_USE_SIRV_RESPONSIVE');

    register_setting( 'sirv-settings-group', 'WP_SIRV_ALWAYS_ADD_HEADERS');

    register_setting( 'sirv-settings-group', 'WP_SIRV_VERSION_PLUGIN_INSTALLED');

    if(get_option('AWS_HOST') !== 's3.sirv.com' || !get_option('AWS_HOST')) update_option('AWS_HOST', 's3.sirv.com');
    if(!get_option('WP_SIRV_NETWORK')) update_option('WP_SIRV_NETWORK', '1');
    if(!get_option('WP_SIRV_CUSTOM_DOMAIN')) update_option('WP_SIRV_CUSTOM_DOMAIN', '');
    if(!get_option('WP_USE_SIRV_RESPONSIVE')) update_option('WP_USE_SIRV_RESPONSIVE', '0');
    if(!get_option('WP_SIRV_ALWAYS_ADD_HEADERS')) update_option('WP_SIRV_ALWAYS_ADD_HEADERS', '0');
    if(!get_option('WP_USE_SIRV_CDN')) update_option('WP_USE_SIRV_CDN', '2');

    $domain = empty($_SERVER['HTTP_HOST']) ? 'MediaLibrary' : $_SERVER['HTTP_HOST'];
    if(!get_option('WP_FOLDER_ON_SIRV')) update_option('WP_FOLDER_ON_SIRV', 'WP_' . $domain);
}

/*add_action( 'upgrader_process_complete', 'sirv_upgrate_function',10, 2);
function sirv_upgrate_function($upgrader_object, $options){
    $current_plugin_path_name = plugin_basename( __FILE__ );

     if ($options['action'] == 'update' && $options['type'] == 'plugin' ){
        foreach($options['plugins'] as $each_plugin){
            if ($each_plugin == $current_plugin_path_name){
                global $wpdb;

                $table_shortcodes_name = $wpdb->prefix . 'sirv_shortcodes';

                $row = $wpdb->get_row( "SELECT * FROM $table_shortcodes_name limit 1", ARRAY_A);

                if(!isset($row['shortcode_options'])){
                    $wpdb->query("ALTER TABLE $table_shortcodes_name ADD COLUMN shortcode_options TEXT NOT NULL after images");
                }


                if(!sirv_is_unique_field('attachment_id')){
                  sirv_set_unique_field('attachment_id');
                }

            }
       }
    }
}*/


function sirv_is_unique_field($field){
  global $wpdb;
  $sirv_images_t = $wpdb->prefix . 'sirv_images';

  $check_data = $wpdb->get_results("SHOW INDEXES FROM $sirv_images_t WHERE Column_name='$field' AND NOT Non_unique", ARRAY_A);

  if(empty($check_data) || $check_data[0]['Non_unique'] == 1) return false; else return true;
}


function sirv_set_unique_field($field){
  global $wpdb;
  $sirv_images_t = $wpdb->prefix . 'sirv_images';
  $duplicated_ids = Array();

  $duplicates_count = $wpdb->get_results("
    SELECT COUNT(t1.id) AS count FROM $sirv_images_t t1
    INNER JOIN $sirv_images_t t2
    WHERE t1.id > t2.id AND t1.$field = t2.$field
    ", ARRAY_A);

  $counter = intval($duplicates_count[0]['count']) >= 1000 ? 1000 : intval($duplicates_count[0]['count']);

  do{
    $duplicated_ids = $wpdb->get_results("
    SELECT t1.id FROM $sirv_images_t t1
    INNER JOIN $sirv_images_t t2
    WHERE t1.id > t2.id AND t1.$field = t2.$field
    LIMIT 1000
    ", ARRAY_A);

    if(!empty($duplicated_ids)){
      $ids = implode("','", array_values(array_unique(sirv_flattern_array($duplicated_ids, true, 'id'))));
      $wpdb->query("DELETE FROM $sirv_images_t WHERE id IN ('$ids')");
    }

    if($counter >= intval($duplicates_count[0]['count'])) break; else $counter += 1000;

  }while(!empty($duplicated_ids));

  $wpdb->query("ALTER TABLE $sirv_images_t ADD UNIQUE ($field)");
}


add_action( 'wp_ajax_sirv_debug', 'sirv_debug_callback' );
function sirv_debug_callback(){
    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    /*$isUniqueField = sirv_is_unique_field('attachment_id');

    if($isUniqueField) echo 'Unique Field'; else sirv_set_unique_field('attachment_id');*/

    wp_die();
}


// create new tab Sirv
//add_filter( 'media_upload_tabs', 'sirv_tab' );

    /*function sirv_tab( $tabs ) {
    $tabs['sirv'] = "Insert from Sirv";
    return $tabs;
}*/


/*// upload scripts, css and iframe with content when tab selected
add_action( 'media_upload_sirv', 'sirv_tab_content' );

function sirv_tab_content() {

    wp_enqueue_style('sirv_style', plugins_url('/sirv/css/wp-sirv.css', __FILE__));
    wp_enqueue_script( 'sirv_logic', plugins_url('/sirv/js/wp-sirv.js', __FILE__), array( 'jquery', 'jquery-ui-sortable' ), '1.0.0');
    wp_localize_script( 'sirv_logic', 'ajax_object', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'assets_path' => plugins_url('/sirv/assets', __FILE__) ) );
    wp_enqueue_script( 'sirv_logic-md5', plugins_url('/sirv/js/wp-sirv-md5.min.js', __FILE__), array(), '1.0.0');

    wp_iframe( 'sirv_frame' );
}


// load iframe
function sirv_frame(){

    include('sirv/template.php');
}*/

if (get_option('WP_SIRV_ALWAYS_ADD_HEADERS') == 1){
   add_action('wp_enqueue_scripts', 'sirv_add_sirv_js', 20);
}
    //action to parse content to find Sirv class in image
    //add_action('shutdown', 'check_Sirv_class', 0);

add_filter( 'the_content'        , 'sirv_check_responsive', 10 );
add_filter( 'get_avatar'         , 'sirv_check_responsive', 10 );
add_filter( 'widget_text'        , 'sirv_check_responsive', 10 );
add_filter( 'get_image_tag'      , 'sirv_check_responsive', 10 );
add_filter( 'post_thumbnail_html', 'sirv_check_responsive', 10 );

function sirv_check_responsive($content){

  if(is_admin()) return $content;

  //echo htmlentities($content);

  $pattern = '/<(img|div).*?class=\".*?Sirv.*?\"/s';
  if(preg_match($pattern, $content) === 1){
    global $sirv_make_responsive;
    $sirv_make_responsive = true;
  }

  //remove BOM symbol
  $content = str_replace("\xEF\xBB\xBF",'',$content);

  return $content;

}


/*ob_start();
function check_Sirv_class() {
    $content = '';

    $levels = ob_get_level();

    for ($i = 0; $i < $levels; $i++) {
        $content .= ob_get_clean();
    }

    if(!is_admin()){
        $pattern = '/<(img|div).*?class=\".*?Sirv.*?\"/s';
        if(preg_match($pattern, $content) === 1){
            if(preg_match('/<script.*?src=[\'\"].*?sirv\.js.*?<\/script>/is', $content) !== 1){
                $content = preg_replace('/(<\/head>)/is', '<script src="https://scripts.sirv.com/sirv.js"></script>$1', $content);
            }
        }
    }

    //remove BOM symbol
    $content = str_replace("\xEF\xBB\xBF",'',$content);

    echo $content;
}*/


// remove http(s) from host in sirv options
add_action( 'admin_notices', 'sirv_check_option');

function sirv_check_option(){
    global $pagenow;
    if ($pagenow == 'admin.php' && $_GET['page'] == SIRV_PLUGIN_PATH .'/sirv/options.php') {
        if ( (isset($_GET['updated']) && $_GET['updated'] == 'true') || (isset($_GET['settings-updated']) && $_GET['settings-updated'] == 'true') ) {
            update_option('AWS_HOST', preg_replace('/(http|https)\:\/\/(.*)/ims', '$2', get_option('AWS_HOST')));
        }
    }
}



add_action('init', 'sirv_init', 20);
function sirv_init(){

    if (get_option('WP_USE_SIRV_CDN') == 1){
        $GLOBALS['sirv_wp_additional_image_sizes'] = isset($GLOBALS['_wp_additional_image_sizes']) ? $GLOBALS['_wp_additional_image_sizes'] : Array();

        add_filter( 'wp_get_attachment_image_src', 'sirv_wp_get_attachment_image_src', 10, 4 );
        add_filter( 'wp_get_attachment_url', 'sirv_wp_get_attachment_url', 10, 2 );
        add_filter( 'wp_calculate_image_srcset', 'sirv_add_custom_image_srcset', 10, 5 );
        add_filter('vc_wpb_getimagesize', 'sirv_vc_wpb_filter', 10,3);
        add_filter('envira_gallery_image_src', 'sirv_envira_crop', 10, 4);
        add_action('wp_enqueue_scripts', 'sirv_enqueue_frontend_scripts', 30);

        add_filter( 'the_content'        , 'sirv_the_content', 10 );
        add_filter( 'get_avatar'         , 'sirv_the_content', 10 );
        add_filter( 'widget_text'        , 'sirv_the_content', 10 );
        add_filter( 'get_image_tag'      , 'sirv_the_content', 10 );
        add_filter( 'post_thumbnail_html', 'sirv_the_content', 10 );

        if (get_option('WP_USE_SIRV_RESPONSIVE') == 1){
            add_filter('wp_get_attachment_image_attributes', 'sirv_do_responsive_images', 99, 3);
        }
    }

}


function sirv_envira_crop($resized_image, $id, $item, $data ){

  if(stripos($resized_image, 'sirv.com') !== false){
    preg_match('/(^http.*)-(\d{2,4})x(\d{2,4})(_[a-z]{1,2})?(\..*)/is', $resized_image, $m);

    $orig_url = '';
    $w = 0;
    $h = 0;
    $isCrop = false;

    if(!empty($m)){
      $orig_url = $m[1] . $m[5];
      $w = $m[2];
      $h = $m[3];
      $isCrop = $m[4] !== '' ? true : false;
    }

    if( $orig_url !== '' && $isCrop ){
      $crop_direction = sirv_crop_direction($m[4]);
      $pattern_crop = '?scale.width=' . $w . '&scale.height=' . $h . '&scale.option=fill&canvas.width=' . $w . '&canvas.height=' . $h;
      $resized_image = $orig_url . $pattern_crop . $crop_direction;
    }
  }

  //return preg_replace('/-\d{2,4}x\d{2,4}(_c)?/is', '', $resized_image);
  return $resized_image;

}


function sirv_crop_direction($type){
  $param_crop_coords = '';

  switch ($type) {
    case '_c':
      $param_crop_coords = '&canvas.position=center';
      break;
    case '_tl':
      $param_crop_coords = '&canvas.position=northeast';
      break;
    case '_tr':
      $param_crop_coords = '&canvas.position=northwest';
      break;
    case '_bl':
      $param_crop_coords = '&canvas.position=southwest';
      break;
    case '_br':
      $param_crop_coords = '&canvas.position=southeast';
      break;
  }

  return $param_crop_coords;
}


function sirv_enqueue_frontend_scripts(){
  wp_enqueue_style('sirv_frontend_style', plugins_url('/sirv/css/sirv-responsive-frontend.css', __FILE__));
  wp_enqueue_script( 'sirv_miscellaneous', plugins_url('/sirv/js/wp-sirv-diff.js', __FILE__), array('jquery'), '1.0.0', true);


  wp_register_script('sirv-js', 'https://scripts.sirv.com/sirv.js', array(), false, true);

  add_action("wp_footer", "sirv_add_src_to_footer", 10);

}


function sirv_add_src_to_footer(){
  global $sirv_make_responsive;

  if(!is_null($sirv_make_responsive) && $sirv_make_responsive){

    wp_enqueue_script( 'sirv-js' );
  }
}


function sirv_add_sirv_js(){
    wp_enqueue_script('sirv-js', 'https://scripts.sirv.com/sirv.js', array(), false, true);
}


function sirv_do_responsive_images($attr, $attachment, $size){

    if(is_admin()) return $attr;

    $attr['class'] = isset($attr['class']) ? $attr['class'] . ' Sirv' : 'Sirv';
    $attr['data-src'] = isset($attr['data-src']) ? $attr['src'] : $attr['src'];
    unset($attr['src']);
    unset($attr['srcset']);
    unset($attr['sizes']);

    return $attr;
}

//-----------------------------------------------------------------------------------------------------
function sirv_the_content($content){

  if (is_admin()) {
    return $content;
  }

  global $wpdb;

  $uploads_dir = wp_upload_dir();
  $root_url_images_path = $uploads_dir['baseurl'];

  $quoted_base_url = preg_replace('/https?\\\:/ims','https?\:',preg_quote($root_url_images_path,'/'));

  preg_match_all('/'.$quoted_base_url.'\/([^\s]*?)(\-[0-9]{1,}(?:x|&#215;)[0-9]{1,})?(\.(?:jpg|jpeg|png|gif))/ims', $content, $m);

  if (!empty($m[0]) && is_array($m[0]) && count($m[0])) {
    $all_image_sizes = sirv_get_image_sizes();
    foreach ($m[0] as $i => $fullURL) {

      $attachment = $wpdb->get_row( $wpdb->prepare(
        "SELECT * FROM $wpdb->postmeta WHERE meta_key = '_wp_attached_file' AND meta_value = %s",
        $m[1][$i].$m[3][$i]
      ), ARRAY_A);

      if (!empty($attachment) && !empty($attachment['post_id'])) {
        $imageURL = '';
        /*if (get_option('WP_USE_SIRV_RESPONSIVE') == 1){
          $pattern = sprintf("/<img.*?(class=\")(.*?\").*?(src=\"%s\").*?(alt=\".*?\").*?(srcset=\".*?\").*?>/i", $m[0][$i]);

          preg_match($pattern, $content, $imgTest);

          print '<br>---------------------------------<br>';
          //print_r($imgTest);
          print_r($m[0][i]);
          print '<br>---------------------------------<br>';

          $resized = wp_get_attachment_image_src($attachment['post_id'], 'full');
          $imageURL = $resized[0];

          $content = preg_replace($pattern, "<img $1 Sirv $2 data-$3 $4>", $content);

          print '<br>---------------------------------<br>';
          echo htmlentities($content);die();
          print '<br>---------------------------------<br>';
        }else */if (empty($m[2][$i])) {
            $resized = wp_get_attachment_image_src($attachment['post_id'], 'full');
            $imageURL = $resized[0];
        } else {
            list($w, $h) = explode('x', str_replace('-', '', str_replace('&#215;', 'x', $m[2][$i])));
            $attachment_meta = wp_get_attachment_metadata( $attachment['post_id'] );
            foreach ($attachment_meta['sizes'] as $size => $size_arr) {
            if ($w==$size_arr['width'] && $h==$size_arr['height']) {
                $resized = wp_get_attachment_image_src($attachment['post_id'], $size);
                $imageURL = $resized[0];
            }
          }
        }
        if ($imageURL!='') {
          $content = str_replace($m[0][$i], $imageURL, $content);
        } else {
        }
      }
    }
  }

  return $content;
}

//------------------------------------------------------------------------------------------------------------------


add_filter( 'wp_prepare_attachment_for_js', 'sirv_wp_prepare_attachment_for_js', 10, 3 );
function sirv_wp_prepare_attachment_for_js($response, $attachment, $meta){
   if (!empty($response['sizes'])) {
    if (preg_match('/^image/ims', $response['type'])) {
      foreach ($response['sizes'] as $size => $image) {
        $response['sizes'][$size]['url'] = preg_replace('/(.*)(?:\-[0-9]{1,}x[0-9]{1,}(\.[a-z]{1,})$)/ims', '$1$2?scale.width='.$image['width'].'&scale.height='.$image['height'], $image['url']);
      }
    }
  }
  return $response;
}


function sirv_wp_get_attachment_image_src($image, $attachment_id, $size, $icon) {

    //disable in admin area
    if(is_admin() || !is_array($image)) return $image;

    require_once 'sirv/options-service.php';
    require_once( ABSPATH . 'wp-admin/includes/file.php' );

    /*$root_images_path = get_home_path() . 'wp-content/uploads';
    $root_url_images_path = get_site_url() . '/wp-content/uploads';*/

    $uploads_dir = wp_upload_dir();
    $root_images_path = $uploads_dir['basedir'];
    $root_url_images_path = $uploads_dir['baseurl'];

    //check if get_option('siteurl') return http or https
    if (stripos(get_option('siteurl'), 'https://') === 0) {
       $root_url_images_path = str_replace('http:','https:',$root_url_images_path);
    }

    $bucket = getValue::getOption('AWS_BUCKET');
    $folder_on_sirv = getValue::getOption('WP_FOLDER_ON_SIRV');

    $image_url = $image[0];
    $image_width = $image[1];
    $image_height = $image[2];

    //clean get params
    $image_url = (stripos($image_url, '?') === false) ? $image_url : preg_replace('/\?.*/is', '', $image_url);

    //fix if image url without site path
    $image_url = sirv_convert_to_corrected_link($image_url);

    $sirv_root_path = sirv_get_domain() . $folder_on_sirv;

    if(stripos(clean_protocol($image_url), clean_protocol($root_url_images_path)) === false) {
        if(stripos($image_url, $sirv_root_path) !== false) {
            $original_image_data = sirv_get_original_image($image_url, $sirv_root_path, $root_images_path);
            $image[0] = sirv_scale_image($original_image_data['original_image_url'], $image_width, $image_height, $size, $original_image_data['original_image_path']);
        }
    }else{
        $cdn_image_url = sirv_get_cdn_image($image_url, $attachment_id, $image_width, $image_height);
        if(!empty($cdn_image_url)){
            $original_image_data = sirv_get_original_image($cdn_image_url, $sirv_root_path, $root_images_path);
            $image[0] = sirv_scale_image($original_image_data['original_image_url'], $image_width, $image_height, $size, $original_image_data['original_image_path']);
        }
    }

    return $image;
}


function clean_protocol($url){
  return preg_replace('/^https?/is', '', $url);
}


function sirv_wp_get_attachment_url($url, $attachment_id ) {
    if(is_admin()) return $url;

    //fix if image url without site path
    $url = sirv_convert_to_corrected_link($url);

    $cdn_image_url = sirv_get_cdn_image($url, $attachment_id);

        if(!empty($cdn_image_url)){
            $url = sirv_get_domain($cdn_image_url);
        }

        $url = addProfile($url);

        return $url;
}


function sirv_add_custom_image_srcset($sources, $size_array, $image_src, $image_meta, $attachment_id){
    //if url not from sirv return original array
    //if(stripos($image_src, 'sirv.com') === false) return $sources;

    if(is_admin() || !is_array($sources)) return $sources;

    require_once( ABSPATH . 'wp-admin/includes/file.php' );

    $bucket = get_option('AWS_BUCKET');
    $folder_on_sirv = get_option('WP_FOLDER_ON_SIRV');
    $uploads_dir = wp_upload_dir();
    $root_images_path = $uploads_dir['basedir'];
    $sirv_root_path = sirv_get_domain() . $folder_on_sirv;

    global $wpdb;
    $table_name = $wpdb->prefix . 'sirv_images';

    $sql_result = $wpdb->get_row("SELECT * FROM " . $table_name ." WHERE attachment_id='".$attachment_id."'", ARRAY_A);

    if(!empty($sql_result)){
        $sql_result['sirv_image_url'] = sirv_get_domain($sql_result['sirv_image_url']);

        $original_image_path = str_replace($sirv_root_path, $root_images_path, $sql_result['sirv_image_url']);
        $image_sizes = array_keys($sources);
        $image_width = '';
        $image_height = '';
        $size_name = null;

        foreach ($image_sizes as $size) {
            if($image_meta['width'] == $size && is_numeric($image_meta['height'])){
              $image_width = $image_meta['width'];
              $image_height = $image_meta['height'];
            }else{
              $size_name = sirv_get_size_name($size, $image_meta['sizes']);
              if(isset($size_name) && !is_null($size_name)){
                  $image_width = $image_meta['sizes'][$size_name]['width'];
                  $image_height = $image_meta['sizes'][$size_name]['height'];
              }else{
                  $image_width = $size;
                  $image_height = $size;
              }
            }

            //$sources[$size]['url'] = sirv_scale_image($sql_result['sirv_image_url'], $size, $size, $size_name, $original_image_path, true);
            $sources[$size]['url'] = sirv_scale_image($sql_result['sirv_image_url'], $image_width, $image_height, $size_name, $original_image_path, true);

        }
    }
    return $sources;
}


function sirv_vc_wpb_filter($img, $img_id, $attributes){

    if(is_admin()) return $img;

    if(in_array($attributes['thumb_size'], array_values(get_intermediate_image_sizes()))) return $img;

    require_once( ABSPATH . 'wp-admin/includes/file.php' );

    $bucket = get_option('AWS_BUCKET');
    $folder_on_sirv = get_option('WP_FOLDER_ON_SIRV');
    $uploads_dir = wp_upload_dir();
    $root_images_path = $uploads_dir['basedir'];
    $sirv_root_path = sirv_get_domain() . $folder_on_sirv;

    preg_match('/(\d{2,4})x(\d{2,4})/is', $attributes['thumb_size'], $sizes);
    $img_sizes = array();
    $img_sizes['width'] = $sizes[1];
    $img_sizes['height'] = $sizes[2];

    $original_image_url = preg_replace('/\?scale.*/is', '', $img['p_img_large'][0]);
    $original_image_path =  str_replace($sirv_root_path, $root_images_path, $original_image_url);

    $scale_pattern = sirv_get_scale_pattern($original_image_path, $img_sizes['width'], $img_sizes['height'], true);
    $img['thumbnail'] = preg_replace('/-'.$sizes[0].'(\.[jpg|jpeg|png|gif]*)/is', '$1'.$scale_pattern, $img['thumbnail']);
    $img['p_img_large'][0] = $original_image_url;

    return $img;

}


function sirv_get_image_size($size){
    $sizes = array();
    $sizes['width'] = get_option( "{$_size}_size_w'");
    $sizes['heigh'] = get_option( "{$_size}_size_h'");
    $sizes['crop'] = (bool)get_option( "{$_size}_crop'");
}

function sirv_get_image_sizes() {
    global $_wp_additional_image_sizes;

    if(!empty($GLOBALS['sirv_wp_additional_image_sizes'])) $_wp_additional_image_sizes = $GLOBALS['sirv_wp_additional_image_sizes'];

    $sizes = array();

    foreach ( get_intermediate_image_sizes() as $_size ) {
        if ( in_array( $_size, array('thumbnail', 'medium', 'medium_large', 'large') ) ) {
            $sizes[ $_size ]['width']  = get_option( "{$_size}_size_w" );
            $sizes[ $_size ]['height'] = get_option( "{$_size}_size_h" );
            $sizes[ $_size ]['crop']   = (bool) get_option( "{$_size}_crop" );
        } elseif ( isset( $_wp_additional_image_sizes[ $_size ] ) ) {
            $sizes[ $_size ] = array(
                'width'  => $_wp_additional_image_sizes[ $_size ]['width'],
                'height' => $_wp_additional_image_sizes[ $_size ]['height'],
                'crop'   => $_wp_additional_image_sizes[ $_size ]['crop'],
            );
        }
    }

    return $sizes;
}


function sirv_get_original_image($image_url, $sirv_root_path, $root_images_path){
    $pattern = '/(.*?)[-|-]\d{1,4}x\d{1,4}(\.[a-zA-Z]{2,5})/is';
    $tested_image = preg_replace($pattern, '$1$2', $image_url);
    $image_path_on_disc = str_replace($sirv_root_path, $root_images_path, $tested_image);
    $orig_image = array();
    if(file_exists($image_path_on_disc)){
        $orig_image['original_image_url'] = $tested_image;
        $orig_image['original_image_path'] = $image_path_on_disc;

    }else{
        $orig_image['original_image_url'] = $image_url;
        $orig_image['original_image_path'] = str_replace($sirv_root_path, $root_images_path, $image_url);
    }
    return $orig_image;
}


function sirv_get_original_sizes($original_image_path){
    $sizes = Array('width' => 0, 'height' => 0);

    $uploads_dir = wp_upload_dir();
    $root_images_path = $uploads_dir['basedir'];

    if(stripos($original_image_path, $root_images_path) !== false){
      $image_dimensions = getimagesize($original_image_path);
      $sizes['width'] = $image_dimensions[0];
      $sizes['height'] = $image_dimensions[1];
    }

    return $sizes;

}


function sirv_scale_image($image_url, $image_width, $image_height, $size, $original_image_path, $isCrop=false){
    $sizes = sirv_get_image_sizes();

    $get_param_symbol = (stripos($image_url, '?') === false) ? '?' : '&';

    //fix if width or height received from sirv_wp_get_attachment_image_src == 0
    if($image_width == 0 || $image_height == 0 || $image_width >= 3000 || $image_height >= 3000){
        if(!empty($sizes) && !is_null($size) && in_array($size, array_keys($sizes))){
            $image_width = $sizes[$size]['width'];
            $image_height = $sizes[$size]['height'];
        }
    }


    $isCrop = false;

    if(in_array($size, array_keys($sizes))){
        $isCrop = (bool)$sizes[$size]['crop'];
    }

    $url = $image_url . sirv_get_scale_pattern($original_image_path, $image_width, $image_height, $isCrop, $get_param_symbol);

    return addProfile($url);
}

function sirv_get_scale_pattern($original_image_path, $image_width, $image_height, $isCrop, $get_param_symbol='?'){
    /*$pattern_crop_portrait = $get_param_symbol.'scale.width='.$image_width.'&crop.width='.$image_width.'&crop.height='.$image_height.'&crop.x=center&crop.y=center';
    $pattern_crop_landscape = $get_param_symbol.'scale.height='.$image_height.'&crop.width='.$image_width.'&crop.height='.$image_height.'&crop.x=center&crop.y=center';
    $pattern_crop_square = $get_param_symbol.'scale.width='.$image_width.'&crop.width='.$image_width.'&crop.height='.$image_height.'&crop.x=center&crop.y=center';*/
    $pattern_crop = $get_param_symbol . 'scale.width=' . $image_width . '&scale.height=' . $image_height . '&scale.option=fill&cw=' . $image_width . '&ch=' . $image_height . '&cx=center&cy=center';
    $pattern_scale = $get_param_symbol.'scale.width='.$image_width .'&scale.height='.$image_height;
    $scale_width = $get_param_symbol.'scale.width=' . $image_width;
    $scale_height = $get_param_symbol.'scale.height=' . $image_height;
    $original = '';

    $usedPattern = '';

    //sometimes wp has strange giant image sizes
    if ($image_width > 3000) return $scale_height;
    if ($image_height > 3000) return $scale_width;
    if ($image_height > 3000 && $image_width > 3000) return '';

    $original_image_sizes = sirv_get_original_sizes($original_image_path);
    if($original_image_sizes['width'] == $image_width && $original_image_sizes['height'] == $image_height) return $original;

    if($isCrop){
        /*$orientation = sirv_test_orientation($original_image_sizes);

        if($orientation == 'landsape'){
            if($image_width <= $image_height) $usedPattern = $pattern_crop_landscape;
            if($image_width > $image_height) $usedPattern = $pattern_crop_portrait;
        }
        if($orientation == 'portrait'){
             if($image_width <= $image_height) $usedPattern = $pattern_crop_portrait;
             if($image_width > $image_height) $usedPattern = $pattern_crop_landscape;
        }
        if($orientation == 'square') $usedPattern = $pattern_crop_square;*/
        $usedPattern = $pattern_crop;
    }else{
        $usedPattern = $pattern_scale;
    }

    return $usedPattern;
}


function sirv_test_orientation($sizes){
    if ($sizes['width'] > $sizes['height']) return 'landsape';
    if ($sizes['width'] < $sizes['height']) return 'portrait';
    if ($sizes['width'] == $sizes['height']) return 'square';
}


function sirv_get_domain($url=''){
    $custom_domain = get_option('WP_SIRV_CUSTOM_DOMAIN');
    $network_type = get_option('WP_SIRV_NETWORK');
    if($url){
        if($network_type == 3 && $custom_domain !==''){
            $parsed_url = parse_url($url);
            return 'https://' . $custom_domain . $parsed_url['path'];
        }else{
            if($network_type == '1') return str_replace('.sirv.com', '-cdn.sirv.com', $url);
            return $url;
        }
    }else{
        if( $network_type == 3 && $custom_domain !== ''){
            return 'https://' . $custom_domain . '/';
        }else{
            $bucket = get_option('AWS_BUCKET');
            return ($network_type == '1') ? "https://{$bucket}-cdn.sirv.com/" : "https://{$bucket}.sirv.com/";
        }

    }
}


function addProfile($url){
    if(stripos($url, 'profile') !== false){
        return $url;
    }

    $profile = get_option('WP_SIRV_CDN_PROFILES');

    if(!empty($profile)){
        $url .= (stripos($url, '?') === false) ? '?profile=' . $profile : '&profile=' . $profile;
    }

    return $url;
}


function sirv_convert_to_corrected_link($image_url){
    $site_url = get_site_url();

    if(stripos($image_url, $site_url) === false){
        if(stripos($image_url, '/wp-content') === 0){
            $image_url = $site_url . $image_url;
        }
    }

    return $image_url;
}


function sirv_get_size_name($size, $array_of_sizes){
    foreach ($array_of_sizes as $size_name_key => $size_name_value) {
        if($size_name_value['width'] == $size) return $size_name_key;
    }

    return null;
}


function encode_spaces($string){
    return str_replace(' ', '%20', $string);
}


function sirv_get_cdn_image($image_url, $attachment_id, $image_width=NULL, $image_height=NULL) {

    require_once( ABSPATH . 'wp-admin/includes/file.php' );

    $uploads_dir = wp_upload_dir();
    $root_images_path = $uploads_dir['basedir'];
    $root_url_images_path = $uploads_dir['baseurl'];

    //check if get_option('siteurl') return http or https
    /*if (stripos(get_option('siteurl'), 'https://') === 0) {
       $root_url_images_path = str_replace('http:','https:',$root_url_images_path);
    }*/
    $image_protocol = preg_match('/^https:/is', $image_url) ? 'https' : 'http';

    $root_url_images_path = $image_protocol . clean_protocol($root_url_images_path);

    global $wpdb;
    $table_name = $wpdb->prefix . 'sirv_images';

    require_once 'sirv/options-service.php';
    $folder_on_sirv = getValue::getOption('WP_FOLDER_ON_SIRV');
    $bucket = getValue::getOption('AWS_BUCKET');

    $image_base_name = str_replace($root_url_images_path, '', $image_url);
    $image_base_path = str_replace(basename($image_base_name), '', $image_base_name);
    $image_path_on_disc = $root_images_path . $image_base_name;
    $sirv_path = $folder_on_sirv . $image_base_path;
    $sirv_full_path = $folder_on_sirv . encode_spaces($image_base_name);

    //exit if file doesn't exist on disc
    if (!file_exists($root_images_path . $image_base_name)) {
        return '';
    }

    $image_size = filesize($root_images_path . $image_base_name);
    $image_created_timestamp = date("Y-m-d H:i:s", filemtime($root_images_path . $image_base_name));

    $sql_result = $wpdb->get_row("
      SELECT * FROM $table_name
      WHERE sirvpath ='$sirv_full_path'
      AND size ='$image_size'
      AND timestamp ='$image_created_timestamp'
      AND attachment_id = $attachment_id
    ", ARRAY_A);

    if(empty($sql_result)){
        require_once 'sirv/aws-s3-helper.php';

        $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());


        //check if image already exists on sirv
        /*$sirv_file_info = $s3object->getObjectInfo($sirv_full_path);
        if(!is_array($sirv_file_info) || empty($sirv_file_info)){*/
        $sirv_image_url_array = $s3object->uploadFile($sirv_full_path, $image_path_on_disc, $web_accessible = true);
        /*}else{
            $sirv_image_url_array = array('full_url' => "https://{$bucket}.sirv.com/" . $sirv_full_path, 'sirv_path' => $sirv_full_path);
        }*/

        if(!empty($sirv_image_url_array)){
            $sirv_image_url = $sirv_image_url_array['full_url'];

            $data = Array();
            $data['attachment_id'] = $attachment_id;
            $data['wp_path'] = $image_base_name;
            $data['size'] = $image_size;
            $data['sirvpath'] = $folder_on_sirv . $image_base_name;
            $data['sirv_image_url'] = $sirv_image_url;
            $data['sirv_folder'] = $folder_on_sirv;
            $data['timestamp'] = $image_created_timestamp;
            $data['timestamp_synced'] = date("Y-m-d H:i:s");

            if($wpdb->get_row("SELECT * FROM ". $table_name . " WHERE attachment_id='".$attachment_id."'")){
                $wpdb->update($table_name, $data, array('attachment_id' => $attachment_id));
            }else{
                $wpdb->insert($table_name, $data);
            }
            return $sirv_image_url;

        }else{
            //return original image
            return '';
        }
    }else{
        //return cached image
        return $sql_result['sirv_image_url'];
    }
}

//return array with images using in posts
function sirv_get_all_images(){
    $query_images_args = array(
        'post_type'      => 'attachment',
        'post_mime_type' => 'image',
        'post_status'    => 'inherit',
        'posts_per_page' => - 1,
    );

    $query_images = new WP_Query( $query_images_args );

    $images = array();
    $images['count'] = count($query_images->posts);
    $tmp_images = array();

    foreach ( $query_images->posts as $image ) {
        $tmp_images[] = array('image_url' => wp_get_attachment_url( $image->ID ), 'attachment_id' => $image->ID);
    }

    $images['images'] = $tmp_images;

    return $images;
}


function sirv_get_unsynced_images($limit=100){

    global $wpdb;
    $sirv_images_t = $wpdb->prefix . 'sirv_images';
    $posts_t = $wpdb->prefix . 'posts';


    /*SELECT wp_posts.ID as attachment_id, wp_posts.guid as attachment_url FROM wp_posts  WHERE 1=1 AND wp_posts.ID NOT IN (SELECT attachment_id FROM wp_sirv_images)
    AND (wp_posts.post_mime_type LIKE 'image/%')
    AND wp_posts.post_type = 'attachment'
    AND ((wp_posts.post_status = 'inherit'))
    ORDER BY wp_posts.post_date DESC LIMIT 10 OFFSET 0*/


    $unsynced_images = $wpdb->get_results("
      SELECT $posts_t.ID as attachment_id, $posts_t.guid as image_url FROM $posts_t
      WHERE $posts_t.ID NOT IN (SELECT attachment_id FROM $sirv_images_t)
      AND ($posts_t.post_mime_type LIKE 'image/%')
      AND $posts_t.post_type = 'attachment'
      AND (($posts_t.post_status = 'inherit'))
      ORDER BY $posts_t.post_date DESC LIMIT $limit
      ", ARRAY_A);



    return $unsynced_images;

}


function sirv_get_all_post_images_count(){
    global $wpdb;
    $posts_t = $wpdb->prefix . 'posts';

    $sql_result = $wpdb->get_results("
        SELECT count(*) as count FROM $posts_t WHERE ($posts_t.post_mime_type LIKE 'image/%')
        AND $posts_t.post_type = 'attachment'
        AND (($posts_t.post_status = 'inherit'))
      ", ARRAY_A);

    //[['count' => 300]]
    return $sql_result;
}


function sirv_get_unsynced_images_count(){
    global $wpdb;
    $sirv_images_t = $wpdb->prefix . 'sirv_images';
    $posts_t = $wpdb->prefix . 'posts';


    $unsynced_images_count = $wpdb->get_results("
      SELECT count(*) as count FROM $posts_t WHERE $posts_t.ID NOT IN (SELECT attachment_id FROM $sirv_images_t)
      AND ($posts_t.post_mime_type LIKE 'image/%')
      AND $posts_t.post_type = 'attachment'
      AND (($posts_t.post_status = 'inherit'))
      ", ARRAY_A);

    return $unsynced_images_count;
}


function sirv_get_uncached_images($post_images){
    global $wpdb;
    $table_name = $wpdb->prefix . 'sirv_images';

    //cached images
    $sql_result = $wpdb->get_results("SELECT attachment_id FROM " . $table_name, ARRAY_N);

    $uncached_ids = array_values(array_diff(sirv_flattern_array($post_images, true, 'attachment_id'), sirv_flattern_array($sql_result)));

    return sirv_get_unique_items($post_images, $uncached_ids);
}


function sirv_get_unique_items($search_array, $unique_items){
    $tmp_arr = array();
    foreach ($search_array as $item) {
        if(in_array($item['attachment_id'], $unique_items)) array_push($tmp_arr, $item);
    }

    return $tmp_arr;
}


function sirv_flattern_array($array, $isAssociativeArray=false, $associativeField=''){
    $tmp_arr = array();
    foreach ($array as $item) {
        if($isAssociativeArray){
            if($associativeField !== ''){
                array_push($tmp_arr, intval($item[$associativeField]));
            }else return array();

        }else array_push($tmp_arr, intval($item[0]));
    }

    return $tmp_arr;
}

//---------------------------------------------YOAST SEO fixes for og images-----------------------------------------------------------------------//

add_filter( 'wpseo_opengraph_image', 'sirv_wpseo_opengraph_image', 10, 1 );
add_filter( 'wpseo_twitter_image', 'sirv_wpseo_opengraph_image', 10, 1 );


function sirv_wpseo_opengraph_image($img){
    if(stripos($img, '-cdn.sirv') != false) $img = str_replace('-cdn', '', $img);

    return $img;
}

//---------------------------------------------YOAST SEO meta fixes for og images END ------------------------------------------------------------------//


//-------------------------------------------------------------Ajax requests-------------------------------------------------------------------------//
function sirv_get_params_array($key=null, $secret_key=null, $bucket=null, $host=null){
    require_once 'sirv/options-service.php';

    $host       = is_null($host) ? 's3.sirv.com' : $host;
    $bucket     = is_null($bucket) ? getValue::getOption('AWS_BUCKET') : $bucket;
    $key        = is_null($key) ? getValue::getOption('AWS_KEY') : $key;
    $secret_key = is_null($secret_key) ? getValue::getOption('AWS_SECRET_KEY') : $secret_key;

    return Array(
        'host'       => $host,
        'bucket'     => $bucket,
        'key'        => $key,
        'secret_key' => $secret_key
    );
}


function sirv_get_cached_data(){
    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_images';
    $table_name_posts = $wpdb->prefix . 'posts';


    //get cahched images data
    //$cached_data = $wpdb->get_results("SELECT COUNT(*) AS count, SUM(size) AS sum_size FROM $table_name", ARRAY_A);
    $cached_data = $wpdb->get_results("
        SELECT
            COUNT(*) AS count,
            SUM(size) AS sum_size,
            (SELECT COUNT(*) FROM $table_name_posts WHERE $table_name_posts.ID = $table_name.attachment_id) AS check_
            FROM $table_name
            GROUP BY check_ HAVING check_ > 0
            ", ARRAY_A);

    if(empty($cached_data)){
        $cached_data = array(array('count' => 0, 'sum_size' => 0));
    }

    return $cached_data;
}


function decode_chunk( $data ) {
    $data = explode( ';base64,', $data );

    if ( ! is_array( $data ) || ! isset( $data[1] ) ) {
        return false;
    }

    $data = base64_decode( $data[1] );
    if ( ! $data ) {
        return false;
    }

    return $data;
}


function checkAndCreatekDir($dir){
    if(!is_dir($dir)) {
        mkdir($dir);
        chmod($dir, 0777);
    }
}


function sirv_destroy_session($session_name){
    session_id($session_name);
    session_start();
    session_unset();
    session_destroy();
}


add_action( 'wp_ajax_sirv_set_session_destroy', 'sirv_set_session_destroy_callback' );
function sirv_set_session_destroy_callback(){

    if(!(is_array($_POST) && isset($_POST['sirv_set_session_destroy']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    sirv_destroy_session('cache-sync-status');

    wp_die();
}


//get sirv storage data
add_action( 'wp_ajax_sirv_get_storage_data', 'sirv_get_storage_data_callback' );
function sirv_get_storage_data_callback(){
    if(!(is_array($_POST) && isset($_POST['sirv_get_storage_data']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    echo $s3object->getStats();

    wp_die();
}


//get sirv traffic data
add_action( 'wp_ajax_sirv_get_traffic_data', 'sirv_get_traffic_data_callback' );
function sirv_get_traffic_data_callback(){
    if(!(is_array($_POST) && isset($_POST['traffic_range']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $from = $_POST['traffic_range'][0];
    $to = $_POST['traffic_range'][1];

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    echo $s3object->getTrafficStats($from, $to);

    wp_die();
}


//get sirv account info
add_action( 'wp_ajax_sirv_get_account_info', 'sirv_get_account_info_callback' );
function sirv_get_account_info_callback(){
    if(!(is_array($_POST) && isset($_POST['sirv_get_account_info']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    echo $s3object->getAccountInfo();

    wp_die();
}



//use ajax request to get php ini variables data
add_action( 'wp_ajax_sirv_get_php_ini_data', 'sirv_get_php_ini_data_callback' );
function sirv_get_php_ini_data_callback(){
    if(!(is_array($_POST) && isset($_POST['sirv_get_php_ini_data']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());
    $accountInfo = json_decode($s3object->getAccountInfo(), true);

    $php_ini_data = array();
    $php_ini_data['post_max_size'] = ini_get('post_max_size');
    $php_ini_data['max_file_uploads'] = ini_get('max_file_uploads');
    $php_ini_data['max_file_size'] = ini_get('upload_max_filesize');
    $php_ini_data['sirv_file_size_limit'] = $accountInfo['account']['fileSizeLimit'];

    echo json_encode($php_ini_data);

    wp_die();
}


//use ajax to clean 30 rows in table. For test purpose.
add_action( 'wp_ajax_sirv_delete_thirty_rows', 'sirv_delete_thirty_rows_callback' );
function sirv_delete_thirty_rows_callback(){
    if(!(is_array($_POST) && isset($_POST['sirv_delete_thirty_rows']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }
    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_images';
    $result = $wpdb->query("DELETE FROM $table_name WHERE id > 0 LIMIT 30");

    echo $result;


    wp_die();
}


//use ajax request to to get all images data used in the posts
add_action( 'wp_ajax_sirv_sync_uncached_images', 'sirv_sync_uncached_images_callback' );
function sirv_sync_uncached_images_callback(){
    if(!(is_array($_POST) && isset($_POST['sirv_sync_uncached_images']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $unsynced_images_count = sirv_get_unsynced_images_count()[0]['count'];

    if($unsynced_images_count == 0) return;

    $all_post_images_count = sirv_get_all_post_images_count()[0]['count'];
    $cached_data = sirv_get_cached_data();
    $image_num = intval($cached_data[0]['count']) + 1;

    //while(true){
    do{

      $unsynced_images = sirv_get_unsynced_images();


      if(!empty($unsynced_images)){
        session_id('cache-sync-status');
        session_start();
        $_SESSION['syncStatusData'] = array();
        $_SESSION['stopSyncStatus'] = false;
        session_write_close();

        $arr_content = array();
        $err_results = array('message' => "Some images couldn't be synced. Please check if it exists and has read access.", 'error_images' => array());
        foreach ($unsynced_images as $image_data) {
            $result = sirv_get_cdn_image($image_data['image_url'], $image_data['attachment_id']);

            if(!empty($result)){
                $arr_content['percent'] = intval($image_num / $all_post_images_count * 100);
                $arr_content['processedImage'] = $image_num;
                $arr_content['count'] = $all_post_images_count;

                session_id('cache-sync-status');
                session_start();
                $_SESSION['syncStatusData'] = $arr_content;
                $_SESSION['stopSyncStatus'] = false;
                session_write_close();
            }else{
                $err_results['error_images'][] = array('image_url' => $image_data['image_url'], 'attachment_id' => $image_data['attachment_id']);
            }

            $image_num++;
        }
      }else{
        session_id('cache-sync-status');
        session_start();
        $_SESSION['stopSyncStatus'] = true;
        session_write_close();
        break;
      }

      //break loop if somesing went wrong
      if($image_num > $all_post_images_count) break;

    } while(!empty($unsynced_images));

/*    $all_post_images = sirv_get_all_images();
    $all_post_images_count = count($all_post_images['images']);
    $uncached_images = sirv_get_uncached_images($all_post_images['images']);
    $uncached_images_count = count($uncached_images);

    $cached_data = sirv_get_cached_data();


    if($uncached_images_count > 0){
        session_id('cache-sync-status');
        session_start();
        $_SESSION['syncStatusData'] = array();
        $_SESSION['stopSyncStatus'] = false;
        session_write_close();

        $arr_content = array();
        $err_results = array('message' => "Some images couldn't be synced. Please check the image exists and has read access.", 'error_images' => array());
        $image_num = intval($cached_data[0]['count']) + 1;
        foreach ($uncached_images as $image_data) {
            $result = sirv_get_cdn_image($image_data['image_url'], $image_data['attachment_id']);

            if(!empty($result)){
                $arr_content['percent'] = intval($image_num/$all_post_images_count * 100);
                $arr_content['processedImage'] = $image_num;
                $arr_content['count'] = $all_post_images_count;

                $image_num++;

                session_id('cache-sync-status');
                session_start();
                $_SESSION['syncStatusData'] = $arr_content;
                $_SESSION['stopSyncStatus'] = false;
                session_write_close();
            }else{
                $err_results['error_images'][] = array('image_url' => $image_data['image_url'], 'attachment_id' => $image_data['attachment_id']);
            }
        }
    }*/

        /*session_id('cache-sync-status');
        session_start();
        $_SESSION['stopSyncStatus'] = true;
        session_write_close();*/

    if(!empty($err_results['error_images'])){
        echo json_encode($err_results);
    }

    wp_die();

}

//monitoring status for creating sirv cache
add_action( 'wp_ajax_sirv_get_sync_status', 'sirv_get_sync_status_callback' );
function sirv_get_sync_status_callback(){

    if(!(is_array($_POST) && isset($_POST['sirv_get_sync_status']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    session_id('cache-sync-status');
    session_start();

    if(empty($_SESSION) || $_SESSION['stopSyncStatus'] === null) $_SESSION['stopSyncStatus'] = true;

    $session_data = $_SESSION['syncStatusData'];
    $session_data['stopSyncStatus'] = $_SESSION['stopSyncStatus'];

    if($_SESSION['stopSyncStatus'] === false){
        if(!empty($_SESSION['syncStatusData'])){
            if (intval($session_data['percent']) === 100) {
                echo json_encode($session_data);

                $_SESSION['syncStatusData'] = array();
                $_SESSION['stopSyncStatus'] = true;

            }else{
                echo json_encode($session_data);
            }
        }else{
            echo json_encode(array("percent" => null, "processedImage" => null, 'count' => null));
        }
    }else{
        $_SESSION['syncStatusData'] = array();
        echo json_encode($session_data);
    }

    session_write_close();

    wp_die();
}


//clear and stop sync upload session
add_action( 'wp_ajax_sirv_clear_upload_session', 'sirv_clear_upload_session_callback' );
function sirv_clear_upload_session_callback(){

    if(!(is_array($_POST) && isset($_POST['sirv_clear_upload_session']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    session_id('cache-sync-status');
    session_start();
    $_SESSION['syncStatusData'] = array();
    $_SESSION['stopSyncStatus'] = true;
    session_write_close();


    wp_die();
}


//use ajax request to get count and size of cached images from sirv_images
add_action( 'wp_ajax_sirv_get_cached_data', 'sirv_get_cached_data_callback' );
function sirv_get_cached_data_callback(){
    if(!(is_array($_POST) && isset($_POST['get_cached_data']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $cached_data = sirv_get_cached_data();

    //get uncached images data count
    $all_post_images = sirv_get_all_images();
    $uncached_images = sirv_get_uncached_images($all_post_images['images']);
    $cached_data[0]['uncached_count'] = count($uncached_images);
    $cached_data[0]['all_post_images'] = count($all_post_images['images']);

    echo json_encode($cached_data[0]);

    wp_die();
}


//ajax request to clear image cache
add_action( 'wp_ajax_sirv_clear_cache', 'sirv_clear_cache_callback' );
function sirv_clear_cache_callback(){
    if(!(is_array($_POST) && isset($_POST['clean_cache_type']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $clean_cache_type = $_POST['clean_cache_type'];

    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_images';

    if($clean_cache_type == 'full'){

        require_once 'sirv/aws-s3-helper.php';

        $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());


        $files = array_column($wpdb->get_results("SELECT sirvpath FROM $table_name", ARRAY_N), 0);
        $result = $s3object->deleteMultipleObjects($files);

    }

    $delete = $wpdb->query("TRUNCATE TABLE $table_name");

    sirv_destroy_session('cache-sync-status');

    echo "successful";

    wp_die();
}


//use ajax request to show data from sirv
add_action( 'wp_ajax_sirv_get_aws_object', 'sirv_get_aws_object_callback' );

function sirv_get_aws_object_callback() {

    if (isset($_POST['path'])){
        $sirv_path = $_POST['path'];
        if (empty($sirv_path)) $sirv_path = '/';
    }

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    $obj = $s3object->getBucketContents(urlencode($sirv_path));


    //alphanumeric sort object by keys(image name)
    $content = $obj["contents"];
    $dirs = $obj['dirs'];
    $dirs_to_remove = array('Profiles/');

    usort($content, function($a, $b){return strnatcasecmp($a['Key'],$b['Key']);});
    usort($dirs, function($a, $b){return strnatcasecmp($a['Prefix'],$b['Prefix']);});

    $obj['contents'] = $content;
    $obj['dirs'] = sirv_remove_dirs($dirs, $dirs_to_remove);

    echo json_encode($obj);

    wp_die(); // this is required to terminate immediately and return a proper response
}

function sirv_remove_dirs($dirs, $dirs_to_remove){
    $tmp_arr = array();
    foreach ($dirs as $key => $dir) {
        if(!in_array($dir['Prefix'], $dirs_to_remove)) {
            $tmp_arr[] = $dir;
        }
    }
    return $tmp_arr;

}


//use ajax to upload images on sirv.com
add_action('wp_ajax_sirv_upload_files', 'sirv_upload_files_callback');

function sirv_upload_files_callback(){

    if(!(is_array($_POST) && is_array($_FILES) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $current_dir = $_POST['current_dir'];
    $current_dir = $current_dir == '/' ? '' : $current_dir;
    $total = intval($_POST['totalFiles']);

    $totalPart = count($_FILES);


    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    $arr_content = array();

    for($i=0; $i<$totalPart; $i++) {

        $filename = $current_dir . basename( $_FILES[$i]["name"]);
        $file = $_FILES[$i]["tmp_name"];

        $result = $s3object->uploadFile($filename, $file, $web_accessible = true, $headers = null);

        session_id('image-uploading-status');
        session_start();

        $image_num = isset($_SESSION['uploadingStatus']['processedImage']) ? $_SESSION['uploadingStatus']['processedImage'] + 1 : 1;

        $arr_content['percent'] = intval($image_num/$total * 100);
        $arr_content['processedImage'] = $image_num;
        $arr_content['count'] = $total;

        $image_num++;

        $_SESSION['uploadingStatus'] = $arr_content;
        session_write_close();

        if(!empty($result)) echo json_encode($result);
    }

    wp_die();

}


//upload big file by chanks
add_action('wp_ajax_sirv_upload_file_by_chanks', 'sirv_upload_file_by_chanks_callback');

function sirv_upload_file_by_chanks_callback(){
    if(!(is_array($_POST) && isset($_POST['binPart']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $arr_content = array();

    $uploads_dir = wp_upload_dir();
    $wp_uploads_dir = $uploads_dir['basedir'];


    $tmp_dir = $wp_uploads_dir . '/tmp_sirv_chunk_uploads/';
    checkAndCreatekDir($tmp_dir);

    $filename = $_POST['partFileName'];
    $binPart = decode_chunk($_POST['binPart']);
    $partNum = $_POST['partNum'];
    $totalParts = $_POST['totalParts'];
    $currentDir = $_POST['currentDir'] == '/' ? '' : $_POST['currentDir'];
    $totalOverSizedFiles =  intval($_POST['totalFiles']);

    $filePath = $tmp_dir . $filename;

    //if($binPart == false) echo 'base64 part cant converted to bin str';

    file_put_contents( $filePath, $binPart, FILE_APPEND );
    chmod($filePath, 0777);


    if($partNum == 1){
        session_id("image-uploading-status");
        session_start();
        $_SESSION['uploadingStatus']['isPartFileUploading'] = true;
        $_SESSION['uploadingStatus']['percent'] = isset($_SESSION['uploadingStatus']['percent']) ? $_SESSION['uploadingStatus']['percent'] : null;
        $_SESSION['uploadingStatus']['processedImage'] = isset($_SESSION['uploadingStatus']['processedImage']) ? $_SESSION['uploadingStatus']['processedImage'] : null;
        $_SESSION['uploadingStatus']['count'] = isset($_SESSION['uploadingStatus']['count']) ? $_SESSION['uploadingStatus']['count'] : null;
        session_write_close();
    }

    if($partNum == $totalParts){

        require_once 'sirv/aws-s3-helper.php';

        $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

        $result = $s3object->uploadFile($currentDir . $filename, $filePath, $web_accessible = true, $headers = null);

        unlink($filePath);

        session_id("image-uploading-status");
        session_start();

        $arr_content['processedImage'] = empty($_SESSION['uploadingStatus']['processedImage']) ? 1 : $_SESSION['uploadingStatus']['processedImage'] + 1;
        $arr_content['count'] = empty($_SESSION['uploadingStatus']['count']) ? $totalOverSizedFiles : $_SESSION['uploadingStatus']['count'];
        $arr_content['percent'] = intval($arr_content['processedImage'] / intval($arr_content['count']) * 100);

        $_SESSION['uploadingStatus'] = $arr_content;
        session_write_close();

        if($arr_content['processedImage'] == $arr_content['count']) echo json_encode(array('stop' => true));
    }

    wp_die();
}


//monitoring status for creating sirv cache
add_action( 'wp_ajax_sirv_get_image_uploading_status', 'sirv_get_image_uploading_status_callback' );
function sirv_get_image_uploading_status_callback(){

    if(!(is_array($_POST) && isset($_POST['sirv_get_image_uploading_status']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

        session_id('image-uploading-status');
        session_start();
        $session_data = isset($_SESSION['uploadingStatus']) ? $_SESSION['uploadingStatus'] : array();

        if(!empty($session_data)){
            if (intval($session_data['percent']) >=100) {
                echo json_encode($session_data);
                session_destroy();
            }else{
                echo json_encode($session_data);
                session_write_close();
            }
        }else{
            session_write_close();
            echo json_encode(array("percent" => null, "processedImage" => null, 'count' => null));
        }

    wp_die();
}


//use ajax to store gallery shortcode in DB
add_action('wp_ajax_sirv_save_shortcode_in_db', 'sirv_save_shortcode_in_db');

function sirv_save_shortcode_in_db(){

    if(!(is_array($_POST) && isset($_POST['shortcode_data']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_shortcodes';

    $data = $_POST['shortcode_data'];
    $data['images'] = serialize($data['images']);
    $data['shortcode_options'] = serialize($data['shortcode_options']);
    $data['timestamp'] = date("Y-m-d H:i:s");

    $wpdb->insert($table_name, $data);

    echo $wpdb->insert_id;


    wp_die();
}


//use ajax to get data from DB by id
add_action('wp_ajax_sirv_get_row_by_id', 'sirv_get_row_by_id');

function sirv_get_row_by_id(){

    if(!(is_array($_POST) && isset($_POST['row_id']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_shortcodes';

    $id = intval($_POST['row_id']);

    $row =  $wpdb->get_row("SELECT * FROM $table_name WHERE id = $id", ARRAY_A);

    $row['images'] = unserialize($row['images']);
    $row['shortcode_options'] = unserialize($row['shortcode_options']);

    echo json_encode($row);

    //echo json_encode(unserialize($row['images']));


    wp_die();
}


//use ajax to get data from DB for shortcodes page
add_action('wp_ajax_sirv_get_shortcodes_data', 'sirv_get_shortcodes_data');

function sirv_get_shortcodes_data(){

    if(!(is_array($_POST) && isset($_POST['shortcodes_page']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $limit = $_POST['itemsPerPage'] ? intval($_POST['itemsPerPage']) : 10;
    $sh_page = intval($_POST['shortcodes_page']);

    global $wpdb;
    $sh_table = $wpdb->prefix . 'sirv_shortcodes';

    $sh_count = $wpdb->get_row("SELECT COUNT(*) AS count FROM $sh_table", ARRAY_A);
    $sh_pages = ceil(intval($sh_count['count']) / $limit);

    if($sh_page > $sh_pages) $sh_page = $sh_pages;

    $offset =  ($sh_page - 1) * $limit;

    $shortcodes =  $wpdb->get_results("
                SELECT *
                FROM $sh_table
                ORDER BY $sh_table.id
                DESC
                LIMIT $limit
                OFFSET $offset
            ", ARRAY_A);

    foreach ($shortcodes as $index => $shortcode) {
      $shortcodes[$index]['images'] = unserialize($shortcode['images']);
      $shortcodes[$index]['shortcode_options'] = unserialize($shortcode['shortcode_options']);
    }

    $tmp_arr = Array('count' => $sh_count['count'], 'shortcodes' => $shortcodes);

    echo json_encode($tmp_arr);


    wp_die();
}


//use ajax to get data from DB for shortcodes page
add_action('wp_ajax_sirv_duplicate_shortcodes_data', 'sirv_duplicate_shortcodes_data');

function sirv_duplicate_shortcodes_data(){
  if(!(is_array($_POST) && isset($_POST['shortcode_id']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $sh_id = intval($_POST['shortcode_id']);

    global $wpdb;
    $sh_table = $wpdb->prefix . 'sirv_shortcodes';

    $data = $wpdb->get_row("
                          SELECT *
                          FROM $sh_table
                          WHERE $sh_table.id = $sh_id
                            ", ARRAY_A);

    unset($data['id']);

    $result = $wpdb->insert($sh_table, $data);

    if($result === 1){
      echo 'Shortcode ID=> '. $sh_id . ' was duplicated';
    }else{
      echo 'Duplication was failed';
    }


    wp_die();

}


//use ajax to delete shortcodes
add_action('wp_ajax_sirv_delete_shortcodes', 'sirv_delete_shortcodes');

function sirv_delete_shortcodes(){
  if(!(is_array($_POST) && isset($_POST['shortcode_ids']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    global $wpdb;

    $sh_table = $wpdb->prefix . 'sirv_shortcodes';

    $shortcode_ids = json_decode($_POST['shortcode_ids']);

    function clean_ids($id){
      return intval($id);
    }

    if(!empty($shortcode_ids)){
      $ids = implode(',', array_map('clean_ids', $shortcode_ids));

      $result = $wpdb->query( "DELETE FROM $sh_table WHERE ID IN($ids)" );

      $msg = $result > 0 ? "Shortcodes were successful delete" : "Something went wrong during deleting shortcodes";
      echo $msg;
    }else{
      echo "Nothing to delete";
    }

    wp_die();
}


//use ajax to save edited shortcode
add_action('wp_ajax_sirv_update_sc', 'sirv_update_sc');

function sirv_update_sc(){

    if(!(is_array($_POST) && isset($_POST['row_id']) && isset($_POST['shortcode_data']) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    global $wpdb;

    $table_name = $wpdb->prefix . 'sirv_shortcodes';

    $id = intval($_POST['row_id']);
    $data = $_POST['shortcode_data'];
    $data['images'] = serialize($data['images']);
    $data['shortcode_options'] = serialize($data['shortcode_options']);


    $row =  $wpdb->update($table_name, $data, array( 'ID' => $id ));

    echo $row;


    wp_die();
}


//use ajax to add new folder in sirv
add_action('wp_ajax_sirv_add_folder', 'sirv_add_folder');

function sirv_add_folder(){

    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $current_dir = $_POST['current_dir'];
    $current_dir = $current_dir == '/' ? '' : $current_dir;
    $new_dir = $_POST['new_dir'];


    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    $s3object->createFolder($current_dir . $new_dir . '/');

    wp_die();
}


//use ajax to check customer login details
add_action( 'wp_ajax_sirv_check_connection', 'sirv_check_connection', 10, 1 );
function sirv_check_connection() {

    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $msg_ok = "Connection: OK";
    $msg_failed = 'Connection failed. Please check your <a href="https://my.sirv.com/#/account/settings" target="_blank">S3 details</a> match.';

    require_once 'sirv/aws-s3-helper.php';

    $host = $_POST['host'];
    $bucket = $_POST['bucket'];
    $key = $_POST['key'];
    $secret_key = $_POST['secret_key'];

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array($key, $secret_key, $bucket, $host));

    $isConnection = $s3object->checkCredentials();

    if($s3object->authMessage){
      echo $s3object->authMessage;
      wp_die();
    }

    $message = $isConnection ? $msg_ok : $msg_failed;

    echo $message;


    wp_die();
}


//use ajax to remove review notice
add_action( 'wp_ajax_sirv_dismiss_review_notice', 'sirv_dismiss_review_notice', 10, 1 );
function sirv_dismiss_review_notice() {

    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    update_option('sirv_review_notice', 'noticed');

    echo 'Sirv review notice dismissed';

    wp_die();
}


function sirv_test_connection($bucket,$key,$secret_key) {
    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array($key, $secret_key, $bucket));

    $isConnection = $s3object->checkCredentials();

    return Array($isConnection, $s3object->authMessage);
}

//use ajax to delete files
add_action( 'wp_ajax_sirv_delete_files', 'sirv_delete_files' );

function sirv_delete_files(){
    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }

    $filenames = $_POST['filenames'];

    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());


    if($s3object->deleteMultipleObjects($filenames)){
        echo 'Files were deleted';
    }else{
        echo 'Files weren\'t deleted';
    }

    wp_die();
}


//use ajax to check if options is empty or not
add_action( 'wp_ajax_sirv_check_empty_options', 'sirv_check_empty_options' );

function sirv_check_empty_options(){

    require_once 'sirv/options-service.php';

    $host = getValue::getOption('AWS_HOST');
    $bucket = getValue::getOption('AWS_BUCKET');
    $key = getValue::getOption('AWS_KEY');
    $secret_key = getValue::getOption('AWS_SECRET_KEY');

    if(empty($host) || empty($bucket) || empty($key) || empty($secret_key)){
        echo false;
    }else{
        echo true;
    }

    wp_die();
}


//use ajax to get sirv profiles
add_action( 'wp_ajax_sirv_get_profiles', 'sirv_get_profiles' );

function sirv_get_profiles(){
    require_once 'sirv/aws-s3-helper.php';

    $s3object = new MagicToolbox_AmazonS3_Helper(sirv_get_params_array());

    $obj = $s3object->getBucketContents('Profiles/');


    echo '<option disabled>Choose profile</option><option value="">Default</option>';
    foreach ($obj["contents"] as $value) {
        $tmp = str_replace('Profiles/', '', $value['Key']);
        if (!empty($tmp)){
            $tmp = basename($tmp, '.profile');
            if($tmp == 'Default') continue;
            echo "<option value='{$tmp}'>{$tmp}</option>";
        }
    }

    wp_die();
}

//use ajax to send message from sirv plugin
add_action( 'wp_ajax_sirv_send_message', 'sirv_send_message' );

function sirv_send_message(){
    if(!(is_array($_POST) && defined('DOING_AJAX') && DOING_AJAX)){
        return;
    }


    //$priority = $_POST['priority'];
    $summary = $_POST['summary'];
    $text = $_POST['text'];
    $name = $_POST['name'];
    $emailFrom = $_POST['emailFrom'];


    $headers = array(
        'From:' . $name . ' <'. $emailFrom . '>'
    );

    //wp_mail( $to, $subject, $message, $headers, $attachments );
    /*echo wp_mail('support@sirv.com', $summary .' - '. $priority, $text, $headers);*/
    echo wp_mail('support@sirv.com', $summary, $text, $headers);

    wp_die();
}


?>
